//  TestThreadHelper.java
//  TestThread
//
//  Created by Anthony Karre on Sun Jan 05 2003.
//  Copyright (c) 2003 Perficient. All rights reserved.
//

package mytask ;

public class TestThreadHelper {

  public String getDemoString () {
  
    // demonstrate correct inner class load
    
    TTHInnerClass myinner = new TTHInnerClass() ;    
    return myinner.getInnerClassString() ;
  }
  
  class TTHInnerClass {
  
    public String getInnerClassString () {
      return "innerclassworked" ;
    }
  
  }

}
