// TestThread.java
// Trivial Thread

package mytask;

import java.util.*;
import mytask.TestThreadHelper ;

public class TestThread extends Thread {

  public TestThread () {
    super() ;
  }

  public void run() {

    System.out.println("TestThread: You are now inside the TestThread thread.");
    System.out.println("TestThread: Accessing another class in this jar...") ;
      
    // Demonstrate the ability to access another class in the
    // same jar file.
      
    try {
	// Add code to
	// perform computation and return the results
	// using a web service call.
      TestThreadHelper tth = new TestThreadHelper() ;      
      System.out.println("TestThread: tth.getDemoString() returned " + tth.getDemoString()) ;
    }
    catch (Exception e) {
      System.out.println("TestThread: Failed to create TestThreadHelper.") ;
      System.out.println(e.toString() + " : " + e.getMessage()) ;
    }
    System.out.println("TestThread: Exiting TestThread thread.") ;
  }
}
